@@ -2,7 +2,7 @@ package FrameNet::WordNet::Detour;
 
 require Exporter;
 our @ISA = qw(Exporter);
-our $VERSION = "0.99";
+our $VERSION = "1.0";
 
 use strict;
 use warnings;
@@ -15,7 +15,7 @@ use WordNet::QueryData;
 use WordNet::Similarity::path;
 use File::Spec;
 
-my $VCACHE = "0.92";
+my $VCACHE = "0.92b";
 
 
 sub new
@@ -360,11 +360,14 @@ sub generate_candidate_frames {
   
   foreach my $candidatesynset (keys %CandidateSynsets) {
     
-    # synonyms and antonyms of candidate synset
-    foreach my $tmpsynset ($self->{'wn'}->querySense("$candidatesynset",'syns'),
-			   $self->{'wn'}->queryWord("$candidatesynset",'ants')) {
-      $AllCandidates{lc("$tmpsynset")} = 1;
-    };
+      # synonyms and antonyms of candidate synset
+      foreach my $tmpsynset ($self->{'wn'}->querySense("$candidatesynset",'syns'),
+			     $self->{'wn'}->queryWord("$candidatesynset",'ants')) {
+	  # HERE !! !!!
+	  
+	  $AllCandidates{lc("$tmpsynset")} = 1 if((! $self->{'limited'}) or
+						  $self->{'synset'} !~ /$tmpsynset/);
+      };
   };
   
   
@@ -372,21 +375,23 @@ sub generate_candidate_frames {
   print STDERR "\n" if ($self->{'verbosity'} gt 1);
   # lookup all candidates
   foreach my $candidatesynset (keys %AllCandidates) {
-    $MatchingFrames = mergeResultHashes($MatchingFrames,$self->lookup_synset($candidatesynset));
-    if ($candidatesynset =~ / /i) {
-      my $synset_with_underscores = $candidatesynset;
-      $synset_with_underscores =~ s/ /_/ig;
-      $MatchingFrames = mergeResultHashes($MatchingFrames,$self->lookup_synset($synset_with_underscores));
-    } elsif ($candidatesynset =~ /_/i) {
-      my $synset_with_spaces = $candidatesynset;
-      $synset_with_spaces =~ s/_/ /ig;
-      $MatchingFrames = mergeResultHashes($MatchingFrames,$self->lookup_synset($synset_with_spaces));
-    }
-    
+      # print STDERR "!!".$candidatesynset."!!\n";
+      next if ($self->{'limited'} and $candidatesynset eq $self->{'synset'});
+      $MatchingFrames = mergeResultHashes($MatchingFrames,$self->lookup_synset($candidatesynset));
+      if ($candidatesynset =~ / /i) {
+	  my $synset_with_underscores = $candidatesynset;
+	  $synset_with_underscores =~ s/ /_/ig;
+	  $MatchingFrames = mergeResultHashes($MatchingFrames,$self->lookup_synset($synset_with_underscores));
+      } elsif ($candidatesynset =~ /_/i) {
+	  my $synset_with_spaces = $candidatesynset;
+	  $synset_with_spaces =~ s/_/ /ig;
+	  $MatchingFrames = mergeResultHashes($MatchingFrames,$self->lookup_synset($synset_with_spaces));
+      }
+      
   };
   
-  $self->{'numberOfSynsets'} = scalar (keys %AllCandidates) - $self->{'limited'};
-  
+  $self->{'numberOfSynsets'} = scalar (keys %AllCandidates);
+  #print STDERR "numberOfSynsets: ".scalar(keys %AllCandidates)."\n";
   print STDERR "\n" if ($self->{'verbosity'});
   
   return $MatchingFrames;
@@ -477,11 +482,13 @@ sub weight_frames {
   my $AllResult;
 
   my $SpreadingFactor;
+  my $FrameSpreading;
 
   foreach my $reason ('lu','match') {
     foreach my $frameName (keys %{$MatchingFrames->{$reason}}) {
       foreach my $fee (keys %{$MatchingFrames->{$reason}->{$frameName}}) {  
 	$SpreadingFactor->{$fee} += 1;
+	$FrameSpreading->{$frameName}++;
       };
     };
   };
@@ -516,8 +523,9 @@ sub weight_frames {
 	
 	$AllResult->{$frameName}->add_weight($weight);
       };
-
-      $AllResult->{$frameName}->weight($AllResult->{$frameName}->weight / $self->{numberOfSynsets});
+      # print STDERR "frameSpreading ($frameName): ".$FrameSpreading->{$frameName}."\n";
+      #$AllResult->{$frameName}->weight($AllResult->{$frameName}->weight /
+	#			       $FrameSpreading->{$frameName});
       
       print STDERR $frameName.
 	"(".(int((($AllResult->{$frameName}->{'weight'})*1000)+.5)/1000).") "
@@ -614,11 +622,15 @@ sub mergeResultHashes {
   return $H1;
 };
 
+sub version {
+    my $self = shift;
+    return $VERSION;
+};
 
 sub round  {
   my $class = shift;
   return int(((shift)*1000)+0.5)/1000;
-}
+};
 
 sub toHash {
   my $list = shift;
@@ -658,7 +670,7 @@ FrameNet::WordNet::Detour - a WordNet to FrameNet Detour.
 
   my $result = $detour->query("walk#v#1");
 
-  if ($result->isOK) {
+  if ($result->is_ok) {
     print "Best frames: \n";
     print join(' ', @{$result->get_best_framenames})."\n";
 
@@ -667,11 +679,11 @@ FrameNet::WordNet::Detour - a WordNet to FrameNet Detour.
 
     print "All frames with weights: \n";
     foreach my $frame (@{$result->get_all_frames}) {
-	print $frame->get_name.": ";
+	print $frame->name.": ";
 	print $frame->get_weight."\n";
     }
   } else {
-    print $result->get_message."\n";
+    print $result->message."\n";
   } 
 
 
@@ -810,9 +822,15 @@ Default: No verbose, no debug.
 
 Returns a rounded number that has only two decimals. Static method.
 
+=item version ( )
+
+Returns the version number of the module.
+
 =back
 
-=head1 BUGS
+=head1 KNOWN BUGS
+
+- More than one detour object accessing the same cache lead to segmentation faults.
 
 Please report bugs to L<mailto:reiter@cpan.org>.
 
